package com.adc.da.vdaims.controller;

import com.adc.da.mp.base.BaseController;
import com.adc.da.mp.base.IBaseService;
import com.adc.da.mp.base.Query;
import com.adc.da.util.http.PageInfo;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import com.adc.da.util.utils.FileUtil;
import com.adc.da.util.utils.StringUtils;
import com.adc.da.util.utils.UserUtil;
import com.adc.da.vdaims.entity.CertConfigSeqTemplateEO;
import com.adc.da.vdaims.entity.VehicleCertEO;
import com.adc.da.vdaims.service.ICertConfigSeqTemplateEOService;
import com.adc.da.vdaims.service.IVehicleCertEOService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

/**
 * <p>
 * 前端控制器
 * </p>
 * 作者 gen
 * 日期 2021-08-31
 */
@RestController
@RequestMapping("/${restPath}/vdaims/certConfigSeqTemplate")
@Api(tags = "|CertConfigSeqTemplateEO|合格证-数据管理-模板库")
public class CertConfigSeqTemplateEOController extends BaseController<CertConfigSeqTemplateEO> {
    private static final Logger logger = LoggerFactory.getLogger(CertConfigSeqTemplateEOController.class);

    @Autowired
    private ICertConfigSeqTemplateEOService service;

    @Autowired
    private IVehicleCertEOService vehicleCertEOService;

    @ApiOperation(value = "导入")
    @PostMapping("/importVehilceCerts")
    @ResponseBody
    public ResponseMessage importTemplate(HttpServletRequest request, @RequestParam("file") MultipartFile file) throws ParseException {
        InputStream is = null;
        try {
            String fileExtension = FileUtil.getFileExtension(file.getOriginalFilename());
            is = file.getInputStream();
            Sheet sheet = null;
            if (fileExtension.endsWith("xls")) {
                Workbook book = new HSSFWorkbook(is);
                sheet = book.getSheetAt(0);
            } else if (fileExtension.endsWith("xlsx")) {
                Workbook book = new XSSFWorkbook(is);
                sheet = book.getSheetAt(0);
            } else {
                return Result.error("请上传Excel格式文件");
            }
            int row = sheet.getLastRowNum();
            Row key = sheet.getRow(1);
            int column = key.getPhysicalNumberOfCells();
            List<VehicleCertEO> vehicleCertEOList = new ArrayList<>();

            for (int i = 2; i <= row; i++) {

                Date date = new Date();
                Row value = sheet.getRow(i);
                if (null != value) {
                    VehicleCertEO vehicleCertByExcel = null;
                    try {

                        vehicleCertByExcel = vehicleCertEOService.getVehicleCertByExcel(key, value);
                    } catch (ParseException e) {
                        return Result.error("日期格式错误");
                    }
                    vehicleCertByExcel.setQyId(UserUtil.getCurrentUserName());
                    vehicleCertByExcel.setHgzStatus(0);


                    vehicleCertEOList.add(vehicleCertByExcel);
                }
            }
            service.importVehilceCerts(vehicleCertEOList);
        } catch (IOException e) {
            return Result.error(e.getMessage());
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                return Result.error(e.getMessage());
            }
        }

        return Result.success();

    }

    @ApiOperation(value = "复制")
    @PostMapping("/copyTemplate/{templateId}")
    @ResponseBody
    public ResponseMessage copyTemplate(HttpServletRequest request, String style, @PathVariable String templateId) {
        return Result.success(service.copyTemplate(templateId, style));

    }

    @ApiOperation(value = "回退")
    @PostMapping("/rebackTemplate/{templateIds}")
    @ResponseBody
    public ResponseMessage rebackTemplate(@PathVariable Serializable... templateIds) {
        return Result.success(service.rebackTemplate(templateIds));

    }


    @ApiOperation(value = "|CertConfigSeqTemplateEO|列表查询", notes = "vdaims:certconfigseqtemplate:list")
    @PostMapping(value = "list", consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:certconfigseqtemplate:list")
    public ResponseMessage<List<CertConfigSeqTemplateEO>> getList(@RequestBody Query<CertConfigSeqTemplateEO> query) {

        return Result.success(super.list(query));
    }

    @ApiOperation(value = "|CertConfigSeqTemplateEO|分页查询", notes = "vdaims:certconfigseqtemplate:page")
    @PostMapping(value = "page", consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:certconfigseqtemplate:page")
    public ResponseMessage<PageInfo<CertConfigSeqTemplateEO>> getPage(@RequestBody Query<CertConfigSeqTemplateEO> query) {

        return Result.success(getPageInfo(super.page(query)));
    }

    @ApiOperation(value = "|CertConfigSeqTemplateEO|详情", notes = "vdaims:certconfigseqtemplate:get")
    @GetMapping("{id}")
    //@RequiresPermissions("vdaims:certconfigseqtemplate:get")
    public ResponseMessage<CertConfigSeqTemplateEO> get(@NotNull @PathVariable Serializable id) {

        return Result.success(super.getById(id));
    }

    @ApiOperation(value = "|CertConfigSeqTemplateEO|新增", notes = "vdaims:certconfigseqtemplate:save")
    @PostMapping(consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:certconfigseqtemplate:save")
    public ResponseMessage<CertConfigSeqTemplateEO> create(@RequestBody CertConfigSeqTemplateEO data) {

        return Result.success(super.save(data));
    }

    @ApiOperation(value = "|CertConfigSeqTemplateEO|修改", notes = "vdaims:certconfigseqtemplate:update")
    @PutMapping(consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:certconfigseqtemplate:update")
    public ResponseMessage<CertConfigSeqTemplateEO> modify(@RequestBody CertConfigSeqTemplateEO data) {

        return Result.success(super.update(data));
    }

    @ApiOperation(value = "|CertConfigSeqTemplateEO|批量删除", notes = "vdaims:certconfigseqtemplate:delete")
    @DeleteMapping("{ids}")
    //@RequiresPermissions("vdaims:certconfigseqtemplate:delete")
    public ResponseMessage delete(@NotNull @PathVariable Serializable... ids) {

        return Result.success(service.removeByIds(Arrays.asList(ids)));
    }

    @ApiOperation(value = "导入校验")
    @PostMapping("/validateVehilceCertsImport")
    @ResponseBody
    public ResponseMessage validateVehilceCertsImport(HttpServletRequest request, @RequestParam("file") MultipartFile file) throws ParseException {
        InputStream is = null;
        try {
            String fileExtension = FileUtil.getFileExtension(file.getOriginalFilename());
            is = file.getInputStream();
            Sheet sheet = null;
            if (fileExtension.endsWith("xls")) {
                Workbook book = new HSSFWorkbook(is);
                sheet = book.getSheetAt(0);
            } else if (fileExtension.endsWith("xlsx")) {
                Workbook book = new XSSFWorkbook(is);
                sheet = book.getSheetAt(0);
            } else {
                return Result.error("请上传Excel格式文件");
            }
            int row = sheet.getLastRowNum();
            Row key = sheet.getRow(1);
            int column = key.getPhysicalNumberOfCells();
            List<VehicleCertEO> vehicleCertEOList = new ArrayList<>();

            for (int i = 2; i <= row; i++) {

                Date date = new Date();
                Row value = sheet.getRow(i);
                if (null != value) {
                    VehicleCertEO vehicleCertByExcel = null;
                    try {

                        vehicleCertByExcel = vehicleCertEOService.getVehicleCertByExcel(key, value);
                    } catch (ParseException e) {
                        return Result.error("日期格式错误");
                    }
                    vehicleCertByExcel.setQyId(UserUtil.getCurrentUserName());
                    vehicleCertByExcel.setHgzStatus(0);


                    vehicleCertEOList.add(vehicleCertByExcel);
                }
            }
            String vins = service.validateVehilceCerts(vehicleCertEOList);
            if (StringUtils.isEmpty(vins)) {
                return Result.success();

            } else {
                return Result.success("0", "导入的" + vins + "与本库存在的vin、发动机编号、制造日期或车身颜色不一致，请核实！如该数据没问题，请点击【确定】", "");

            }
        } catch (IOException e) {
            return Result.error(e.getMessage());
        } finally {
            try {
                is.close();
            } catch (IOException e) {
                return Result.error(e.getMessage());
            }
        }

//        return Result.success();

    }


    @Override
    protected IBaseService<CertConfigSeqTemplateEO> getService() {

        return service;
    }

    @Override
    protected void verifyQuery(Query<CertConfigSeqTemplateEO> queryItem) {

    }

    @Override
    protected void verifySaveData(CertConfigSeqTemplateEO data) {

    }

    @Override
    protected void verifyUpdateData(CertConfigSeqTemplateEO data) {

    }

    @Override
    protected void verifyDeleteId(Serializable... ids) {

    }
}
