package com.adc.da.vdaims.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.adc.da.mp.base.BaseController;
import com.adc.da.mp.base.IBaseService;
import com.adc.da.mp.base.Query;
import com.adc.da.mp.base.QueryItem;
import com.adc.da.util.http.PageInfo;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import com.adc.da.util.utils.StringUtils;
import com.adc.da.util.utils.UserUtil;
import com.adc.da.vdaims.entity.CertPrintAllEO;
import com.adc.da.vdaims.exception.BusinessException;
import com.adc.da.vdaims.service.ICertPrintAllEOService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.hibernate.validator.constraints.NotEmpty;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

/**
 * <p>
 * 前端控制器
 * </p>
 * 作者 gen
 * 日期 2021-09-13
 */
@RestController
@RequestMapping("/${restPath}/vdaims/certPrintAll")
@Api(tags = "|CertPrintAllEO|一键打印")
public class CertPrintAllEOController extends BaseController<CertPrintAllEO> {
    private static final Logger logger = LoggerFactory.getLogger(CertPrintAllEOController.class);

    @Autowired
    private ICertPrintAllEOService service;

    @ApiOperation(value = "|CertPrintAllEO|列表查询", notes = "vdaims:certprintall:list")
    @PostMapping(value = "list", consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:certprintall:list")
    public ResponseMessage<List<CertPrintAllEO>> getList(@RequestBody Query<CertPrintAllEO> query) {

        return Result.success(super.list(query));
    }

    @ApiOperation(value = "|CertPrintAllEO|分页查询", notes = "vdaims:certprintall:page")
    @PostMapping(value = "page", consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:certprintall:page")
    public ResponseMessage<PageInfo<CertPrintAllEO>> getPage(@RequestBody Query<CertPrintAllEO> query) {
        String QyId = UserUtil.getQyId();

        List<QueryItem<CertPrintAllEO>> querys = query.getQuerys();
        if (CollectionUtil.isEmpty(querys)) {
            querys = new ArrayList<>();
            querys.add(new QueryItem<>());
        }
        QueryItem<CertPrintAllEO> queryItem = querys.get(0);
        CertPrintAllEO eq = StringUtils.isEmpty(queryItem.getEq()) ? new CertPrintAllEO() : queryItem.getEq();
        eq.setQyId(QyId);
        queryItem.setEq(eq);
        return Result.success(getPageInfo(super.page(query)));
    }

    @ApiOperation(value = "|CertPrintAllEO|详情", notes = "vdaims:certprintall:get")
    @GetMapping("{id}")
    //@RequiresPermissions("vdaims:certprintall:get")
    public ResponseMessage<CertPrintAllEO> get(@NotNull @PathVariable Serializable id) {

        return Result.success(super.getById(id));
    }

    @ApiOperation(value = "|CertPrintAllEO|新增", notes = "vdaims:certprintall:save")
    @PostMapping(consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:certprintall:save")
    public ResponseMessage<CertPrintAllEO> create(@RequestBody CertPrintAllEO data) {

        return Result.success(super.save(data));
    }

    @ApiOperation(value = "|CertPrintAllEO|修改", notes = "vdaims:certprintall:update")
    @PutMapping(consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:certprintall:update")
    public ResponseMessage<CertPrintAllEO> modify(@RequestBody CertPrintAllEO data) {

        return Result.success(super.update(data));
    }

    @ApiOperation(value = "|CertPrintAllEO|批量删除", notes = "vdaims:certprintall:delete")
    @DeleteMapping("{ids}")
    //@RequiresPermissions("vdaims:certprintall:delete")
    public ResponseMessage delete(@NotNull @PathVariable Serializable... ids) {

        return Result.success(super.deleteBatchIds(ids));
    }

    @ApiOperation(value = "申请退回")
    @PostMapping(value = "applyBack",consumes = APPLICATION_JSON_VALUE)
    public ResponseMessage batchApplyBack(@NotEmpty @RequestBody List<String> idList) {
        try {
            service.batchApplyBack(idList);
        } catch (BusinessException be) {
            return Result.error(be.getMessage());
        }
        return Result.success();
    }

    @Override
    protected IBaseService<CertPrintAllEO> getService() {

        return service;
    }

    @Override
    protected void verifyQuery(Query<CertPrintAllEO> queryItem) {

    }

    @Override
    protected void verifySaveData(CertPrintAllEO data) {

    }

    @Override
    protected void verifyUpdateData(CertPrintAllEO data) {

    }

    @Override
    protected void verifyDeleteId(Serializable... ids) {

    }
}
