package com.adc.da.vdaims.controller;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;
import javax.validation.constraints.NotNull;

import cn.hutool.core.collection.CollectionUtil;
import com.adc.da.mp.base.IBaseService;
import com.adc.da.mp.base.Query;
import com.adc.da.util.http.PageInfo;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import com.adc.da.vdaims.entity.CertZkEO;
import com.adc.da.vdaims.entity.ChpdpkZkEO;
import com.adc.da.vdaims.entity.ClcpZkEO;
import com.adc.da.vdaims.service.ICertZkEOService;
import com.adc.da.vdaims.service.IChpdpkZkEOService;
import com.adc.da.vdaims.service.IClcpZkEOService;
import com.adc.da.vdaims.util.StringUtil;
import com.adc.da.vdaims.vo.CertZkVO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;
import org.springframework.web.bind.annotation.RequestMapping;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;
import com.adc.da.mp.base.BaseController;

/**
 * <p>
 *  前端控制器
 * </p>
 * 作者 gen
 * 日期 2021-09-10
 */
@RestController
@RequestMapping("/${restPath}/vdaims/certZk")
@Api(tags = "|CertZkEO|合格证-数据管理--新")
public class CertZkEOController extends BaseController<CertZkEO> {
    private static final Logger logger = LoggerFactory.getLogger(CertZkEOController.class);

    @Autowired
    private ICertZkEOService service;

    @Autowired
    private IChpdpkZkEOService chpdpkZkEOService;

    @Autowired
    private IClcpZkEOService clcpZkEOService;

    @ApiOperation(value = "|ClcpZkEO|整车获取下拉框数据", notes = "vdaims:clcpzk:get")
    @GetMapping("/zk/getByConfigSeq/{configSeqId}")
    //@RequiresPermissions("vdaims:clcpzk:get")
    public ResponseMessage<ClcpZkEO> getVehicleZKByConfigSeq(@NotNull @PathVariable Serializable configSeqId) {
        ClcpZkEO clcpZkEO = clcpZkEOService.getVehicleZKByConfigSeq(configSeqId);
        if (clcpZkEO==null){
            clcpZkEO=new ClcpZkEO();
        }else{
            clcpZkEO.setMfadongji(StringUtil.isEmpty(clcpZkEO.getMfadongji()) ? "" : clcpZkEO.getMfadongji().replace(";", ","));
            clcpZkEO.setMgonglv(StringUtil.isEmpty(clcpZkEO.getMgonglv()) ? "" : clcpZkEO.getMgonglv().replace(";", ","));
            clcpZkEO.setMpailiang(StringUtil.isEmpty(clcpZkEO.getMpailiang()) ? "" : clcpZkEO.getMpailiang().replace(";", ","));
            clcpZkEO.setYouhao(StringUtil.isEmpty(clcpZkEO.getYouhao()) ? "" : clcpZkEO.getYouhao().replace(";", ","));

        }
        return Result.success(clcpZkEO);
    }

    @ApiOperation(value = "|ClcpZkEO|底盘获取下拉框数据", notes = "vdaims:clcpzk:get")
    @GetMapping("/dp/getByConfigSeq/{configSeqId}")
    //@RequiresPermissions("vdaims:clcpzk:get")
    public ResponseMessage<ChpdpkZkEO> getDPZKByConfigSeq(@NotNull @PathVariable Serializable configSeqId) {
        ChpdpkZkEO chpdpkZkEO = chpdpkZkEOService.getByConfigSeq(configSeqId);
        if (null==chpdpkZkEO){
            chpdpkZkEO=new ChpdpkZkEO();
        }
        return Result.success(chpdpkZkEO);
    }

    @ApiOperation(value = "|ClcpZkEO|根据整车底盘ID获取下拉框数据", notes = "vdaims:clcpzk:get")
    @GetMapping("/dp/{dpid}")
    //@RequiresPermissions("vdaims:clcpzk:get")
    public ResponseMessage<ChpdpkZkEO> getByDpid(@NotNull @PathVariable Serializable dpid) {

        QueryWrapper<ChpdpkZkEO> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("DPID",dpid);
        queryWrapper.orderByDesc(" to_number(批次)");
        List<ChpdpkZkEO> list = chpdpkZkEOService.list(queryWrapper);
        ChpdpkZkEO chpdpkZkEO = CollectionUtil.isEmpty(list) ?new ChpdpkZkEO() :list.get(0);
        return Result.success(chpdpkZkEO);
    }


    @ApiOperation(value = "|CertZkEO|详情", notes = "vdaims:certzk:get")
    @GetMapping("{id}")
    //@RequiresPermissions("vdaims:certzk:get")
    public ResponseMessage<CertZkEO> get(@NotNull @PathVariable Serializable id) {

        return Result.success(service.getByConfigSeq(id));
    }

//    @ApiOperation(value = "|CertZkEO|新增", notes = "vdaims:certzk:save")
//    @PostMapping(consumes = APPLICATION_JSON_VALUE)
//    //@RequiresPermissions("vdaims:certzk:save")
//    public ResponseMessage<CertZkEO> create(@RequestBody CertZkEO data) {
//
//        return Result.success(super.save(data));
//    }

    @ApiOperation(value = "|CertZkEO|修改", notes = "vdaims:certzk:update")
    @PutMapping(consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:certzk:update")
    public ResponseMessage<CertZkEO> modify(@RequestBody CertZkVO data) {

        return Result.success(service.saveOrUpdate(data.getCertZkEO(),data.getCongfigSeqId()));
    }

    @ApiOperation(value = "|CertZkEO|批量删除", notes = "vdaims:certzk:delete")
    @DeleteMapping("{ids}")
    //@RequiresPermissions("vdaims:certzk:delete")
    public ResponseMessage delete(@NotNull @PathVariable Serializable... ids) {

         return Result.success(super.deleteBatchIds(ids));
    }

    @Override
    protected IBaseService<CertZkEO> getService() {

        return service;
    }

    @Override
    protected void verifyQuery(Query<CertZkEO> queryItem) {

    }

    @Override
    protected void verifySaveData(CertZkEO data) {

    }

    @Override
    protected void verifyUpdateData(CertZkEO data) {

    }

    @Override
    protected void verifyDeleteId(Serializable... ids) {

    }
}
