package com.adc.da.vdaims.controller;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;
import javax.validation.Valid;
import javax.validation.constraints.NotNull;

import com.adc.da.mp.base.IBaseService;
import com.adc.da.mp.base.Query;
import com.adc.da.util.http.PageInfo;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import com.adc.da.vdaims.entity.CertZkExtEO;
import com.adc.da.vdaims.service.ICertZkExtEOService;
import com.adc.da.vdaims.vo.CertZkExtVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

import org.springframework.web.bind.annotation.RequestMapping;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;
import com.adc.da.mp.base.BaseController;

/**
 * <p>
 * 前端控制器
 * </p>
 * 作者 gen
 * 日期 2021-08-25
 */
@RestController
@RequestMapping("/${restPath}/vdaims/certZkExt")
@Api(tags = "|CertZkExtEO|合格证-其他信息")
public class CertZkExtEOController extends BaseController<CertZkExtEO> {
    private static final Logger logger = LoggerFactory.getLogger(CertZkExtEOController.class);

    @Autowired
    private ICertZkExtEOService service;


    @ApiOperation(value = "|CertZkExtEO|详情", notes = "vdaims:certzkext:get")
    @GetMapping("{id}")
    //@RequiresPermissions("vdaims:certzkext:get")
    public ResponseMessage<CertZkExtEO> get(@NotNull @PathVariable Serializable id) {

        return Result.success(super.getById(id));
    }

    @ApiOperation(value = "|CertZkExtEO|根据配置序列号查询", notes = "vdaims:certzkext:get")
    @GetMapping("/getByConfigSeq/{configSeqId}")
    //@RequiresPermissions("vdaims:certzkext:get")
    public ResponseMessage<CertZkExtEO> getByConfigSeq(@NotNull @PathVariable Serializable configSeqId) {

        return Result.success(service.getByConfigSeqId(configSeqId));
    }

    @ApiOperation(value = "|CertZkExtEO|新增", notes = "vdaims:certzkext:save")
    @PostMapping(consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:certzkext:save")
    public ResponseMessage<CertZkExtEO> create(@RequestBody CertZkExtEO data) {
        service.checkSave(data);
        return Result.success(super.save(data));
    }

    @ApiOperation(value = "|CertZkExtEO|修改或提交", notes = "vdaims:certzkext:update")
    @PutMapping(consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:certzkext:update")
    public ResponseMessage<CertZkExtEO> modify(@Validated @RequestBody CertZkExtVO data) {
        service.checkUpdate(data.getCertZkExtEO());
        return Result.success(service.saveOrUpdate(data.getCertZkExtEO(), data.getCongfigSeqId(), data.isSubmit()));
    }

    @ApiOperation(value = "|CertZkExtEO|批量删除", notes = "vdaims:certzkext:delete")
    @DeleteMapping("{ids}")
    //@RequiresPermissions("vdaims:certzkext:delete")
    public ResponseMessage delete(@NotNull @PathVariable Serializable... ids) {

        return Result.success(super.deleteBatchIds(ids));
    }

    @Override
    protected IBaseService<CertZkExtEO> getService() {

        return service;
    }

    @Override
    protected void verifyQuery(Query<CertZkExtEO> queryItem) {

    }

    @Override
    protected void verifySaveData(CertZkExtEO data) {

    }

    @Override
    protected void verifyUpdateData(CertZkExtEO data) {

    }

    @Override
    protected void verifyDeleteId(Serializable... ids) {

    }
}
