package com.adc.da.vdaims.controller;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.Serializable;
import java.util.*;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotNull;

import com.adc.da.enums.ConfigSeqEnum;
import com.adc.da.mp.base.IBaseService;
import com.adc.da.mp.base.Query;
import com.adc.da.mp.base.QueryItem;
import com.adc.da.sys.entity.UserEO;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.http.PageInfo;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import com.adc.da.util.utils.*;
import com.adc.da.util.utils.UUID;
import com.adc.da.vdaims.entity.CertZkEO;
import com.adc.da.vdaims.entity.ConfigSeqEO;
import com.adc.da.vdaims.service.ICertZkEOService;
import com.adc.da.vdaims.service.IConfigSeqEOService;
import com.adc.da.vdaims.util.StringUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import com.adc.da.mp.base.BaseController;
import org.springframework.web.multipart.MultipartFile;

/**
 * <p>
 * 前端控制器
 * </p>
 * 作者 gen
 * 日期 2021-08-23
 */
@RestController
@RequestMapping("/${restPath}/vdaims/configSeq")
@Api(tags = "合格证-数据导入")
public class ConfigSeqEOController extends BaseController<ConfigSeqEO> {
    private static final Logger logger = LoggerFactory.getLogger(ConfigSeqEOController.class);

    @Autowired
    private IConfigSeqEOService service;

    @Autowired
    private ICertZkEOService certZkEOService;

    @Value("${tempdir}")
    private String TEMPDIR;

    @ApiOperation(value = "|ConfigSeqEO|列表查询", notes = "vdaims:configseq:list")
    @PostMapping(value = "list", consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:configseq:list")
    public ResponseMessage<List<ConfigSeqEO>> getList(@RequestBody Query<ConfigSeqEO> query) {
        return Result.success(super.list(query));
    }

    @ApiOperation(value = "|ConfigSeqEO|数据导入", notes = "vdaims:configseq:list")
    @PostMapping(value = "/import")
    public ResponseMessage<List<ConfigSeqEO>> importCSV(@RequestParam MultipartFile file, HttpServletRequest request) {
        long startTime = System.currentTimeMillis();
        if (!FileUtil.getFileExtension(file.getOriginalFilename()).equalsIgnoreCase("csv")) {
            return Result.error("导入文件格式不正确，导入失败");
        }
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.MILLISECOND, 0);
        Date uploadDate = calendar.getTime();
        String uploadDateStr = DateUtils.dateToString(uploadDate, "yyyy-MM-dd");
        String oldFileName = file.getOriginalFilename();
        String fileName = "configsequence" + uploadDateStr + ".csv";
        if (!FileUtil.exists(TEMPDIR)) {
            //如果不存在创建目录
            FileUtil.mkdir(TEMPDIR);
        }
        String path = TEMPDIR + fileName;
        ConfigSeqEO configSeqEO = new ConfigSeqEO();
        try (FileOutputStream out = new FileOutputStream(path)) {
            InputStream is = file.getInputStream();

            //创建一个缓冲区
            byte[] buffer = new byte[1024];
            //判断输入流中的数据是否已经读完的标志
            int len = 0;
            while ((len = is.read(buffer)) > 0) {
                out.write(buffer, 0, len);
            }
            //关闭输入流
            is.close();

            long fileEndTime = System.currentTimeMillis();
            String loggerMsg = "导入完成，用时(秒)： " + (fileEndTime - startTime) / 1000;
            logger.info(loggerMsg);
            List<LinkedHashMap<String, Object>> linkedHashMaps = CSVUtils.csvReader(service.getCsvTitle(), new FileInputStream(path), path);
            service.importCsv(linkedHashMaps);
        } catch (Exception e) {
            logger.error("导入配置序列号失败：", e);
            return Result.error(e.getMessage());
        }
        return Result.success();
    }

    @ApiOperation(value = "|ConfigSeqEO|批量提交", notes = "vdaims:configseq:delete")
    @PostMapping("/submit/{ids}")
    //@RequiresPermissions("vdaims:configseq:delete")
    public ResponseMessage submit(@NotNull @PathVariable Serializable... ids) {
        return Result.success(service.submit(ids));
    }

    @ApiOperation(value = "|ConfigSeqEO|分页查询", notes = "vdaims:configseq:page")
    @PostMapping(value = "page", consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:configseq:page")
    public ResponseMessage<PageInfo<ConfigSeqEO>> getPage(@RequestBody Query<ConfigSeqEO> query) {

        return Result.success(getPageInfo(super.page(query)));
    }

    @ApiOperation(value = "|ConfigSeqEO|数据管理-分页查询", notes = "vdaims:configseq:page")
    @PostMapping(value = "dataManagePage", consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:configseq:page")
    public ResponseMessage<PageInfo<ConfigSeqEO>> getDataManagePage(@RequestBody Query<ConfigSeqEO> query) {
        PageInfo<ConfigSeqEO> pageInfo = getPageInfo(super.page(query));
        List<ConfigSeqEO> configSeqEOList = pageInfo.getList();
        if (CollectionUtils.isNotEmpty(configSeqEOList)) {
//            List<String> configIds = list.stream().map(ConfigSeqEO::getId).collect(Collectors.toList());
            for (ConfigSeqEO configSeqEO : configSeqEOList) {
                QueryWrapper<CertZkEO> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq(CertZkEO.CERT_ZK_CONFIG_SEQ_ID, configSeqEO.getId());
                CertZkEO certZkEO = certZkEOService.getOne(queryWrapper);
                if (null == certZkEO) {
                    configSeqEO.setUpdateTime(null);
                } else {
                    configSeqEO.setUpdateTime(certZkEO.getUpdateTime());
                }
            }

        }
        return Result.success(pageInfo);
    }

    @ApiOperation(value = "|ConfigSeqEO|详情", notes = "vdaims:configseq:get")
    @GetMapping("{id}")
    //@RequiresPermissions("vdaims:configseq:get")
    public ResponseMessage<ConfigSeqEO> get(@NotNull @PathVariable Serializable id) {

        return Result.success(service.getById(id));
    }


    @ApiOperation(value = "|ConfigSeqEO|新增", notes = "vdaims:configseq:save")
    @PostMapping(consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:configseq:save")
    public ResponseMessage<ConfigSeqEO> create(@RequestBody ConfigSeqEO data) {

        return Result.success(super.save(data));
    }

    @ApiOperation(value = "|ConfigSeqEO|修改", notes = "vdaims:configseq:update")
    @PutMapping(consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:configseq:update")
    public ResponseMessage<ConfigSeqEO> modify(@RequestBody ConfigSeqEO data) {

        return Result.success(super.update(data));
    }

    @ApiOperation(value = "|ConfigSeqEO批量修改", notes = "vdaims:configseq:update")
    @PutMapping(value = "modifBatch/{ids}")
    //@RequiresPermissions("vdaims:configseq:update")
    public ResponseMessage<ConfigSeqEO> modifyBatch(@RequestBody ConfigSeqEO data, @NotNull @PathVariable Serializable... ids) {
        QueryWrapper<ConfigSeqEO> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(ConfigSeqEO.CERT_CONFIG_SEQ_ID, ids);
        List<ConfigSeqEO> list = service.list(queryWrapper);
        if (CollectionUtils.isNotEmpty(list)){
            for (ConfigSeqEO configSeqEO : list) {
                if(configSeqEO.getType().equals(1)&&data.getVehicleType().contains("底盘")){
//                    整车。所选车辆类型为底盘进行提示
                    throw new AdcDaBaseException(configSeqEO.getProductModel()+"车型不属于底盘，请重新选择");
                }
                if(configSeqEO.getType().equals(2)&&!data.getVehicleType().contains("底盘")){
//                    底盘。所选车辆类型为整车进行提示
                    throw new AdcDaBaseException(configSeqEO.getProductModel()+"车型不属于底盘，请重新选择");
                }
            }
            service.update(data, queryWrapper);
        }
        return Result.success();
    }

    @ApiOperation(value = "|ConfigSeqEO|回退", notes = "vdaims:configseq:update")
    @PostMapping(value = "/rollback/{ids}")
    //@RequiresPermissions("vdaims:configseq:update")
    public ResponseMessage rollback(@NotNull @PathVariable Serializable... ids) {

        return Result.success(service.rollBack(ids));
    }


    @ApiOperation(value = "|ConfigSeqEO|批量删除", notes = "vdaims:configseq:delete")
    @DeleteMapping("{ids}")
    //@RequiresPermissions("vdaims:configseq:delete")
    public ResponseMessage delete(@NotNull @PathVariable Serializable... ids) {

        return Result.success(super.deleteBatchIds(ids));
    }

    @Override
    protected IBaseService<ConfigSeqEO> getService() {

        return service;
    }

    @Override
    protected void verifyQuery(Query<ConfigSeqEO> queryItem) {

    }

    @Override
    protected void verifySaveData(ConfigSeqEO data) {

    }

    @Override
    protected void verifyUpdateData(ConfigSeqEO data) {


    }

    @Override
    protected void verifyDeleteId(Serializable... ids) {

    }
}
