package com.adc.da.vdaims.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.adc.da.mp.base.BaseController;
import com.adc.da.mp.base.IBaseService;
import com.adc.da.mp.base.Query;
import com.adc.da.mp.base.QueryItem;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.http.PageInfo;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import com.adc.da.util.utils.UserUtil;
import com.adc.da.vdaims.entity.VehicleCertEO;
import com.adc.da.vdaims.service.IVehicleCertEOService;
import com.adc.da.vdaims.util.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

/**
 * <p>
 * 前端控制器
 * </p>
 * 作者 gen
 * 日期 2021-08-24
 */
@RestController
@RequestMapping("/${restPath}/vdaims/vehicleCert")
@Api(tags = "|VehicleCertEO|")
public class VehicleCertEOController extends BaseController<VehicleCertEO> {
    private static final Logger logger = LoggerFactory.getLogger(VehicleCertEOController.class);

    @Autowired
    private IVehicleCertEOService service;

    @ApiOperation(value = "提交确认")
    @PostMapping("/submit/{ids}")
    @ResponseBody
    public ResponseMessage submit(@PathVariable Serializable... ids) {
        return Result.success(service.submit(ids));
    }

    @ApiOperation(value = "一键打印·合格证退回")
    @PostMapping("/returnHgz/{vins}")
    @ResponseBody
    public ResponseMessage returnHgz(@PathVariable Serializable... vins) {
        return Result.success(service.returnHgz(vins));
    }

    @ApiOperation(value = "|VehicleCertEO|列表查询", notes = "vdaims:vehiclecert:list")
    @PostMapping(value = "list", consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:vehiclecert:list")
    public ResponseMessage<List<VehicleCertEO>> getList(@RequestBody Query<VehicleCertEO> query) {
        List<QueryItem<VehicleCertEO>> querys = query.getQuerys();
        if (CollectionUtil.isEmpty(querys)) {
            querys = new ArrayList<>();
            querys.add(new QueryItem<>());
        }
        QueryItem<VehicleCertEO> queryItem = querys.get(0);
        VehicleCertEO eq = queryItem.getEq();
        eq.setCreator(UserUtil.getCurrentUserId());
        queryItem.setEq(eq);
        return Result.success(super.list(query));
    }

    @ApiOperation(value = "|VehicleCertEO|分页查询", notes = "vdaims:vehiclecert:page")
    @PostMapping(value = "page", consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:vehiclecert:page")
    public ResponseMessage<PageInfo<VehicleCertEO>> getPage(@RequestBody Query<VehicleCertEO> query) {

        List<QueryItem<VehicleCertEO>> querys = query.getQuerys();
        if (CollectionUtil.isEmpty(querys)) {
            querys = new ArrayList<>();
            querys.add(new QueryItem<>());
        }
        QueryItem<VehicleCertEO> queryItem = querys.get(0);
        VehicleCertEO eq = queryItem.getEq();
        eq.setCreator(UserUtil.getCurrentUserId());
        queryItem.setEq(eq);
        return Result.success(getPageInfo(super.page(query)));
    }

    @ApiOperation(value = "|VehicleCertEO|详情", notes = "vdaims:vehiclecert:get")
    @GetMapping("{id}")
    //@RequiresPermissions("vdaims:vehiclecert:get")
    public ResponseMessage<VehicleCertEO> get(@NotNull @PathVariable Serializable id) {

        return Result.success(super.getById(id));
    }

    @ApiOperation(value = "|VehicleCertEO|新增", notes = "vdaims:vehiclecert:save")
    @PostMapping(consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:vehiclecert:save")
    public ResponseMessage<VehicleCertEO> create(@RequestBody VehicleCertEO data) {
        if (data.getVin().trim().length() != 17) {
            throw new AdcDaBaseException("车辆识别代号长度应为17位");
        }
        if (StringUtil.isEmpty(data.getHgzCertno()) && StringUtil.isEmpty(data.getDphgzCertno())) {
            throw new AdcDaBaseException("合格证编号和底盘合格证编号不能同时为空");
        }

        return Result.success(service.modify(data));
    }

    @ApiOperation(value = "|VehicleCertEO|修改", notes = "vdaims:vehiclecert:update")
    @PutMapping(consumes = APPLICATION_JSON_VALUE)
    //@RequiresPermissions("vdaims:vehiclecert:update")
    public ResponseMessage<VehicleCertEO> modify(@RequestBody VehicleCertEO data) {
        service.checkUpdate(data);
        if (data.getVin().trim().length() != 17) {
            throw new AdcDaBaseException("车辆识别代号长度应为17位");
        }
        if (StringUtil.isEmpty(data.getHgzCertno()) && StringUtil.isEmpty(data.getDphgzCertno())) {
            throw new AdcDaBaseException("合格证编号和底盘合格证编号不能同时为空");
        }
        return Result.success(super.update(data));
    }

    @ApiOperation(value = "|VehicleCertEO|批量删除", notes = "vdaims:vehiclecert:delete")
    @DeleteMapping("{vins}")
    //@RequiresPermissions("vdaims:vehiclecert:delete")
    public ResponseMessage delete(@NotNull @PathVariable Serializable... vins) {
        return Result.success(service.removeByVin(vins));

    }

    @Override
    protected IBaseService<VehicleCertEO> getService() {

        return service;
    }

    @Override
    protected void verifyQuery(Query<VehicleCertEO> queryItem) {

    }

    @Override
    protected void verifySaveData(VehicleCertEO data) {
//        if (StringUtil.isEmpty(data.getVin())) {
//            throw new AdcDaBaseException("VIN长度应为17位");
//        }
//        if (StringUtil.isEmpty(data.getHgzCertno())) {
//            throw new AdcDaBaseException("合格证编号不能为空");
//        }
//        if (data.getVin().trim().length() != 17) {
//            throw new AdcDaBaseException("VIN长度应为17位");
//        }
//
//        QueryWrapper<VehicleCertEO> queryWrapper3 = new QueryWrapper<>();
//        queryWrapper3.eq(VehicleCertEO.VEHICLE_CERT_HGZ_CERTNO, data.getHgzCertno());
//        if (CollectionUtil.isNotEmpty(service.list(queryWrapper3))) {
//            throw new AdcDaBaseException("合格证编号" + data.getHgzCertno() + "已存在");
//        }
    }

    @Override
    protected void verifyUpdateData(VehicleCertEO data) {
//        if (StringUtil.isEmpty(data.getVin())) {
//            throw new AdcDaBaseException("VIN长度应为17位");
//        }
//        if (data.getVin().trim().length() != 17) {
//            throw new AdcDaBaseException("VIN长度应为17位");
//        }
//        if (StringUtil.isEmpty(data.getHgzCertno())) {
//            throw new AdcDaBaseException("合格证编号不能为空");
//        }
//
//
//        QueryWrapper<VehicleCertEO> queryWrapper3 = new QueryWrapper<>();
//        queryWrapper3.eq(VehicleCertEO.VEHICLE_CERT_HGZ_CERTNO, data.getHgzCertno());
//        queryWrapper3.ne(VehicleCertEO.VEHICLE_CERT_ID, data.getId());
//        if (CollectionUtil.isNotEmpty(service.list(queryWrapper3))) {
//            throw new AdcDaBaseException("合格证编号" + data.getHgzCertno() + "已存在");
//        }
    }

    @Override
    protected void verifyDeleteId(Serializable... ids) {

    }
}
