package com.adc.da.vdaims.controller;

import com.adc.da.excel.poi.excel.ExcelImportUtil;
import com.adc.da.excel.poi.excel.entity.ImportParams;
import com.adc.da.mp.base.BaseController;
import com.adc.da.mp.base.IBaseService;
import com.adc.da.mp.base.Query;
import com.adc.da.mp.base.QueryItem;
import com.adc.da.util.http.PageInfo;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.ResponseMessageCodeEnum;
import com.adc.da.util.http.Result;
import com.adc.da.util.utils.*;
import com.adc.da.vdaims.annotation.TableAuthCheck;
import com.adc.da.vdaims.entity.VehicleCertEO;
import com.adc.da.vdaims.entity.YzxBaseDataEO;
import com.adc.da.vdaims.entity.YzxTemplateDataEO;
import com.adc.da.vdaims.enums.DeclareStatus;
import com.adc.da.vdaims.enums.ParamType;
import com.adc.da.vdaims.enums.PrintStatus;
import com.adc.da.vdaims.enums.TemplateType;
import com.adc.da.vdaims.enums.ConfirmStatus;
import com.adc.da.vdaims.exception.BusinessException;
import com.adc.da.vdaims.handler.PageHandler;
import com.adc.da.vdaims.service.IVehicleCertEOService;
import com.adc.da.vdaims.service.IYzxBaseDataEOService;
import com.adc.da.vdaims.service.IYzxTemplateDataEOService;
import com.adc.da.vdaims.util.FileUtils;
import com.adc.da.vdaims.util.ResourceUtil;
import com.adc.da.vdaims.vo.VehicleCertYzxVo;
import com.adc.da.vdaims.vo.YzxTemplateInfoVo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.io.File;
import java.io.Serializable;
import java.net.URLEncoder;
import java.util.*;
import java.util.concurrent.*;
import java.util.stream.Collectors;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

/**
 * <p>
 * 前端控制器
 * </p>
 * 作者 gen
 * 日期 2021-08-25
 */
@RestController
@RequestMapping("/${restPath}/vdaims/yzx")
@Api(tags = "一致性")
public class YzxBaseDataEOController extends BaseController<YzxBaseDataEO> {
    private static final Logger logger = LoggerFactory.getLogger(YzxBaseDataEOController.class);

    @Autowired
    private IYzxBaseDataEOService service;
    @Autowired
    private IVehicleCertEOService vehicleCertEOService;
    @Autowired
    private IYzxTemplateDataEOService yzxTemplateDataEOService;
    private ConcurrentHashMap<String, Long> qyIdCache = new ConcurrentHashMap<>();

    @ApiOperation(value = "|YzxBaseDataEO|基础数据分页查询")
    @PostMapping(value = "basePage", consumes = APPLICATION_JSON_VALUE)
    public ResponseMessage<PageInfo<YzxBaseDataEO>> getPage(HttpServletRequest request, @RequestBody Query<YzxBaseDataEO> query) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        String qyId = usermap.get("USERNAME").toString();
        QueryItem<YzxBaseDataEO> queryItem = query.getQuerys().get(0);
        queryItem.setEq(new YzxBaseDataEO().setQyId(qyId));
        return Result.success(getPageInfo(super.page(query)));
    }

    @ApiOperation(value = "基础数据详情")
    @GetMapping("{cocId}")
    public ResponseMessage get(@NotNull @PathVariable String cocId) {
        return Result.success(service.getOne(new QueryWrapper<YzxBaseDataEO>().eq(YzxBaseDataEO.COC_ID, cocId).eq(YzxBaseDataEO.QY_ID, UserUtil.getQyId()), false));
    }

    @TableAuthCheck(service = IYzxBaseDataEOService.class, paramName = "data.id", paramType = ParamType.string)
    @ApiOperation(value = "基础数据信息修改")
    @PutMapping(consumes = APPLICATION_JSON_VALUE)
    public ResponseMessage<YzxBaseDataEO> modify(@RequestBody YzxBaseDataEO data) {
        if (StringUtils.isNotBlank(data.getCarCategoryl())) {
            data.setAutoType(data.getCarCategoryl());
        }
        return Result.success(super.update(data));
    }

    @ApiOperation(value = "基础数据批量更新确认状态")
    @PostMapping(value = "batchBaseConfirm", consumes = APPLICATION_JSON_VALUE)
    public ResponseMessage batchBaseConfirm(@RequestBody List<String> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            Result.error("基础数据丢失");
        }
        Collection<YzxBaseDataEO> yzxBaseDataEOS = service.listByIds(idList);
        for (YzxBaseDataEO baseDataEO : yzxBaseDataEOS) {
            if (TemplateType.getByValue(baseDataEO.getTemplateType()) == TemplateType.NONE) {
                return Result.error("一致性证书编号为" + baseDataEO.getCocId() + "的基础数据未选择模板，请先选择模板！");
            }
        }
        service.update(new UpdateWrapper<YzxBaseDataEO>().set(YzxBaseDataEO.CONFIRM_STATUS, ConfirmStatus.CONFIRMED.getValue())
                .set(YzxBaseDataEO.UPDATE_TIME, new Date())
                .set(YzxBaseDataEO.UPDATER, UserUtil.getCurrentUserId())
                .in(YzxBaseDataEO.ID, idList)
                .eq(YzxBaseDataEO.QY_ID, UserUtil.getQyId()));
        return Result.success();
    }

    @ApiOperation(value = "基础数据批量取消确认状态")
    @PostMapping(value = "batchBaseCancel", consumes = APPLICATION_JSON_VALUE)
    public ResponseMessage batchBaseCancel(@RequestBody List<String> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            Result.error("基础数据丢失");
        }
        try {
            service.batchBaseCancel(idList);
        } catch (BusinessException be) {
            return Result.error(be.getMessage());
        }
        return Result.success();
    }

    @ApiOperation(value = "|VehicleCertYzxVo|一致性证书分页查询")
    @PostMapping(value = "certPage")
    public ResponseMessage<PageInfo> getYzxPage(HttpServletRequest request, @RequestBody Query<VehicleCertYzxVo> query) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        String qyId = usermap.get("USERNAME").toString();
        Page<VehicleCertYzxVo> page = new Page<>(query.getPageNo(), query.getPageSize());
        QueryWrapper queryWrapper = query.createWrapper();
        return Result.success(PageHandler.getPageInfo(service.selectYzxInfoList(page, queryWrapper, qyId)));
    }

    @ApiOperation(value = "同步一致性数据")
    @PostMapping(value = "syncYzxData/{batch}")
    public ResponseMessage syncYzxData(HttpServletRequest request, @PathVariable("batch") String batch) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        String name = usermap.get("NAME").toString();
        String qyId = usermap.get("USERNAME").toString();
        RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
        // 缓存的企业ID同步间隔设置为3分钟
        Long times = qyIdCache.getOrDefault(qyId, 0L);
        if ((System.currentTimeMillis() - times) < 180_000L) {
            return Result.success();
        }
        CompletableFuture<Void> future = CompletableFuture.runAsync(() -> {
            service.syncYzxData(name, qyId, batch, requestAttributes);
        });
        qyIdCache.put(qyId, System.currentTimeMillis());
        try {
            // 异步方法，先跑10秒同步一部分数据然后直接返回
            future.get(10, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        } catch (TimeoutException e) {
            e.printStackTrace();
        }
        return Result.success();
    }

    @TableAuthCheck(service = IVehicleCertEOService.class, paramName = "idList", paramType = ParamType.list_string)
    @ApiOperation(value = "|YzxBaseDataEO|批量更新确认状态")
    @PostMapping(value = "batchConfirm", consumes = APPLICATION_JSON_VALUE)
    public ResponseMessage batchConfirm(@RequestBody List<String> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            Result.error("一致性证书ID为空");
        }
        yzxTemplateDataEOService.update(new UpdateWrapper<YzxTemplateDataEO>().set(YzxTemplateDataEO.CONFIRM_STATUS, 1)
                .set(YzxTemplateDataEO.UPDATE_TIME, new Date())
                .set(YzxTemplateDataEO.UPDATER, UserUtil.getCurrentUserId())
                .in(YzxTemplateDataEO.ID, idList));
        return Result.success();
    }

    @ApiOperation(value = "|YzxBaseDataEO|模板数据导入")
    @PostMapping(value = "import")
    public ResponseMessage importTemplateExcel(HttpServletRequest request, @RequestParam("file") MultipartFile file) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        String qyId = usermap.get("USERNAME").toString();
        String fileExtension = FileUtil.getFileExtension(file.getOriginalFilename());
        if (!ConstantUrils.EXCEL_TYPE_XLS.equalsIgnoreCase(fileExtension) && !ConstantUrils.EXCEL_TYPE_XLSX.equalsIgnoreCase(fileExtension)) {
            return Result.error("导入文件格式不正确，导入失败");
        }
        ImportParams params = new ImportParams();
        params.setTitleRows(1);
        params.setHeadRows(1);
        try {
            List<YzxTemplateInfoVo> templateInfoVos = ExcelImportUtil.importExcel(file.getInputStream(), YzxTemplateInfoVo.class, params);
            if (CollectionUtils.isEmpty(templateInfoVos)) {
                return Result.error("请填写模板数据");
            }
            for (YzxTemplateInfoVo templateInfoVo : templateInfoVos) {
                if (StringUtils.isBlank(StringUtils.trim(templateInfoVo.getVin())) || StringUtils.trim(templateInfoVo.getVin()).length() != 17) {
                    return Result.error("车辆识别码输入错误");
                }
                if (StringUtils.isBlank(StringUtils.trim(templateInfoVo.getFdjNo()))) {
                    return Result.error("发动机号不能为空");
                }
                if (StringUtils.isBlank(StringUtils.trim(templateInfoVo.getVehicleColor()))) {
                    return Result.error("车身颜色不能为空");
                }
                if (templateInfoVo.getProduceDate() == null) {
                    return Result.error("制造日期不能为空");
                }
                if (StringUtils.isBlank(StringUtils.trim(templateInfoVo.getYzxCertno()))) {
                    return Result.error("COC证书编号不能为空");
                }
            }
            return Result.success(ResponseMessageCodeEnum.SUCCESS.getCode(), service.importYzxData(templateInfoVos, qyId));
        } catch (BusinessException e) {
            return Result.error(e.getMessage());
        } catch (Exception e) {
            logger.error("导入一致性模板数据失败", e);
            return Result.error("导入一致性模板数据失败");
        }
    }

    @ApiOperation(value = "生成word打印文档")
    @PostMapping(value = "createWords", consumes = APPLICATION_JSON_VALUE)
    public ResponseMessage createWords(@NotNull @RequestBody List<String> vehicleCertIdList, @RequestParam(value = "isPdfView", required = false) boolean isPdfView) {
        try {
            return Result.success(service.createWords(vehicleCertIdList, isPdfView, false));
        } catch (BusinessException be) {
            return Result.error(be.getMessage());
        }
    }

    @ApiOperation(value = "重新生成word打印文档（仅限已打印记录）")
    @PostMapping(value = "declare/createWords", consumes = APPLICATION_JSON_VALUE)
    public ResponseMessage reCreateWords(@NotNull @RequestBody List<String> vehicleCertIdList, @RequestParam(value = "isPdfView", required = false) boolean isPdfView) {
        try {
            return Result.success(service.createWords(vehicleCertIdList, isPdfView, true));
        } catch (BusinessException be) {
            return Result.error(be.getMessage());
        }
    }

    @ApiOperation(value = "下载文档")
    @GetMapping(value = "download")
    public ResponseMessage download(HttpServletResponse response, @RequestParam(value = "fileName") String fileName) {
        try {
            String filePath = ResourceUtil.getConfigByName("previewPath");
            File file = new File(filePath + fileName);
            if (!file.exists()) {
                return Result.error("文件不存在");
            }
            response.setCharacterEncoding("UTF-8");
            response.setHeader("Content-Disposition",
                    "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            String extend = FileUtils.getExtend(fileName);
            if (extend.equals("text")) {
                response.setHeader("content-Type", "text/plain");
            } else if (extend.equals("doc") || extend.equals("docx")) {
                response.setHeader("content-Type", "application/msword");
            } else if (extend.equals("xls")) {
                response.setHeader("content-Type", "application/vnd.ms-excel");
            } else if (extend.equals("pdf")) {
                response.setHeader("content-Type", "application/pdf");
            } else if (extend.equals("jpg") || extend.equals("jpeg")) {
                response.setHeader("content-Type", "image/jpeg");
            } else {
                response.setHeader("content-Type", "application/x-msdownload");
            }
            byte[] data = org.apache.commons.io.FileUtils.readFileToByteArray(file);
            response.getOutputStream().write(data);
            return Result.success();
        } catch (Exception e) {
            logger.error("下载文件失败：", e);
            return Result.error("下载文件失败");
        }
    }

    @TableAuthCheck(service = IVehicleCertEOService.class)
    @ApiOperation(value = "修改打印状态")
    @PutMapping(value = "modifyPrintStatus/{id}/{status}")
    public ResponseMessage modifyPrintStatus(@PathVariable("id") String id, @PathVariable("status") String status) {
        PrintStatus updPrintStatus = PrintStatus.getByValue(status);
        if (updPrintStatus == PrintStatus.NONE) {
            return Result.error("打印状态错误");
        }
        VehicleCertEO vehicleCertEO = vehicleCertEOService.getById(id);
        if (vehicleCertEO == null) {
            logger.error("VEHICLE_CERT的数据丢失：id=" + id);
            return Result.error("一致性证书数据丢失");
        }
        YzxTemplateDataEO yzxTemplateDataEO = yzxTemplateDataEOService.getById(id);
        if (yzxTemplateDataEO == null) {
            logger.error("YZX_TEMPLATE_DATA的数据丢失：id=" + id);
            return Result.error("一致性证书模板数据丢失");
        }
        if (PrintStatus.PRINTED.getValue().equals(yzxTemplateDataEO.getPrintStatus())) {
            return Result.error("VIN编号为" + vehicleCertEO.getVin() + "的一致性证书已打印,状态无法修改");
        }

        Date sysDate = new Date();
        String userId = UserUtil.getCurrentUserId();
        yzxTemplateDataEO.setPrintStatus(updPrintStatus.getValue())
                .setUpdateTime(sysDate)
                .setUpdater(userId);
        // 已打印,将数据修改为未申报
        if (updPrintStatus == PrintStatus.PRINTED) {
            yzxTemplateDataEO.setDeclareStatus(DeclareStatus.NOT_DECLARE.getValue());
        }
        yzxTemplateDataEOService.updateById(yzxTemplateDataEO);
        return Result.success();
    }

    @ApiOperation(value = "获取基本数据模板类型选择列表")
    @GetMapping("/getSelectOptions/{id}")
    public ResponseMessage getTmpSelectOptions(@PathVariable(value = "id") String id) {
        YzxBaseDataEO yzxBaseDataEO = service.getById(id);
        if (yzxBaseDataEO == null) {
            logger.error("YZX_BASE_DATA的数据丢失：id=", id);
            return Result.error("基础数据丢失");
        }
        List<TemplateType> types = Arrays.stream(TemplateType.values()).filter(x -> {
            return x.getAutoType().contains(yzxBaseDataEO.getAutoType()) && x.getCocAutoType().equals(yzxBaseDataEO.getVehicleType());
        }).collect(Collectors.toList());
        return Result.success(types);
    }

    @ApiOperation(value = "修改基础数据模板类型")
    @PostMapping("/modifyTemplateType/{id}/{type}")
    public ResponseMessage modifyTemplateType(@PathVariable(value = "id") String id, @NotBlank @PathVariable(value = "type") String type) {
        YzxBaseDataEO yzxBaseDataEO = service.getById(id);
        if (yzxBaseDataEO == null) {
            logger.error("YZX_BASE_DATA的数据丢失：id=", id);
            return Result.error("基础数据丢失");
        }
        if (ConfirmStatus.CONFIRMED.getValue().equals(yzxBaseDataEO.getConfirmStatus())) {
            return Result.error("基础数据已确认，模板无法修改");
        }
        TemplateType templateType = TemplateType.getByValue(type);
        if (templateType == TemplateType.NONE) {
            logger.error("选择模板类型不存在,type=" + type);
            return Result.error("选择模板不存在，请联系管理员");
        }
        List<TemplateType> types = Arrays.stream(TemplateType.values()).filter(x -> {
            return x.getAutoType().contains(yzxBaseDataEO.getAutoType()) && x.getCocAutoType().equals(yzxBaseDataEO.getVehicleType());
        }).collect(Collectors.toList());
        if (!types.contains(templateType)) {
            return Result.error("模板与车辆类型不匹配，请重新确认！");
        }
        service.update(new UpdateWrapper<YzxBaseDataEO>().set(YzxBaseDataEO.TEMPLATE_TYPE, type).eq(YzxBaseDataEO.ID, id));
        return Result.success();
    }

    @ApiOperation(value = "批量取消确认")
    @PostMapping(value = "batchCancelConfirm")
    public ResponseMessage batchCancelConfirm(@NotEmpty @RequestBody List<String> vinList) {
        List<VehicleCertEO> vehicleCertEOList = vehicleCertEOService.lambdaQuery().in(VehicleCertEO::getVin, vinList).eq(VehicleCertEO::getQyId, UserUtil.getQyId()).list();
        List<String> idList = new ArrayList<>();
        vehicleCertEOList.forEach(x->idList.add(x.getId()));
        Collection<YzxTemplateDataEO> yzxTemplateDataEOs = CollectionUtils.isEmpty(idList)? new ArrayList<>() : yzxTemplateDataEOService.listByIds(idList);
        if (CollectionUtils.isEmpty(yzxTemplateDataEOs)) {
            return Result.error("数据不存在");
        }
        for (YzxTemplateDataEO dataEO : yzxTemplateDataEOs) {
            if (PrintStatus.PRINTED.getValue().equals(dataEO.getPrintStatus())) {
                return Result.error("已打印的数据无法修改确认状态");
            }
        }
        yzxTemplateDataEOService.update(new UpdateWrapper<YzxTemplateDataEO>().set(YzxTemplateDataEO.CONFIRM_STATUS, ConfirmStatus.NOT_CONFIRM.getValue())
                .set(YzxTemplateDataEO.PRINT_STATUS, PrintStatus.NOT_PRINT.getValue()).in(YzxTemplateDataEO.ID, idList));
        return Result.success();
    }

    @Override
    protected IBaseService<YzxBaseDataEO> getService() {

        return service;
    }

    @Override
    protected void verifyQuery(Query<YzxBaseDataEO> queryItem) {

    }

    @Override
    protected void verifySaveData(YzxBaseDataEO data) {

    }

    @Override
    protected void verifyUpdateData(YzxBaseDataEO data) {

    }

    @Override
    protected void verifyDeleteId(Serializable... ids) {

    }

}
