package com.adc.da.vdaims.controller;

import java.io.IOException;
import java.io.Serializable;
import java.util.*;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import javax.xml.ws.WebServiceException;
import com.adc.da.mp.base.IBaseService;
import com.adc.da.mp.base.Query;
import com.adc.da.util.http.ResponseMessage;
import com.adc.da.util.http.Result;
import com.adc.da.util.utils.ExcelUtils;
import com.adc.da.util.utils.StringUtils;
import com.adc.da.util.utils.UserUtil;
import com.adc.da.vdaims.annotation.TableAuthCheck;
import com.adc.da.vdaims.service.IYzxBaseDataEOService;
import com.adc.da.vdaims.webservice.vinupload.ManuRelationBean;
import com.adc.da.vdaims.entity.VehicleCertEO;
import com.adc.da.vdaims.webservice.vinupload.VinBean;
import com.adc.da.vdaims.entity.YzxTemplateDataEO;
import com.adc.da.vdaims.enums.ConfirmStatus;
import com.adc.da.vdaims.enums.DeclareStatus;
import com.adc.da.vdaims.enums.PrintStatus;
import com.adc.da.vdaims.enums.TemplateType;
import com.adc.da.vdaims.enums.VinUploadRes;
import com.adc.da.vdaims.exception.BusinessException;
import com.adc.da.vdaims.handler.YzxTemplateHandler;
import com.adc.da.vdaims.service.IVehicleCertEOService;
import com.adc.da.vdaims.webservice.vinupload.IVinUploadService;
import com.adc.da.vdaims.service.IYzxTemplateDataEOService;
import com.adc.da.vdaims.util.WebServiceUtil;
import com.adc.da.vdaims.vo.CocInfoamtionVo;
import com.adc.da.vdaims.vo.BaseTemplateVo;
import com.adc.da.vdaims.vo.VinUploadVo;
import com.adc.da.vdaims.vo.YzxTemplateExcelVo;
import com.adc.da.vdaims.vo.VehicleCertYzxVo;
import com.adc.da.vdaims.vo.YzxDeclareExcelVo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.poi.ss.usermodel.Workbook;
import org.hibernate.validator.constraints.NotBlank;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.RequestMapping;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;
import com.adc.da.mp.base.BaseController;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

/**
 * <p>
 * 一致性模板数据 前端控制器
 * </p>
 * 作者 gen
 * 日期 2021-09-01
 */
@RestController
@RequestMapping("/${restPath}/vdaims/yzxTemplate")
@Api(tags = "|YzxTemplateDataEO|一致性模板数据")
public class YzxTemplateDataEOController extends BaseController<YzxTemplateDataEO> {
    private static final Logger logger = LoggerFactory.getLogger(YzxTemplateDataEOController.class);
    static ObjectMapper objectMapper = new ObjectMapper();
    @Autowired
    private IYzxTemplateDataEOService service;
    @Autowired
    private IVehicleCertEOService vehicleCertEOService;
    @Autowired
    private IYzxBaseDataEOService yzxBaseDataEOService;

    @ApiOperation(value = "|YzxTemplateDataEO|一致性模板详细数据")
    @GetMapping("{vehicleCertId}/{templateType}")
    public ResponseMessage getTemplate(@NotBlank @PathVariable(value = "vehicleCertId") String vehicleCertId, @NotBlank @PathVariable(value = "templateType") String templateType) {
        BaseTemplateVo templateVo = null;
        try {
            templateVo = service.getTemplate(vehicleCertId, templateType);
        } catch (BusinessException be) {
            return Result.error(be.getMessage());
        }
        return Result.success(templateVo);
    }

    @TableAuthCheck(service = IVehicleCertEOService.class, paramName = "vehicleCertId")
    @ApiOperation(value = "|YzxTemplateDataEO|一致性模板详细数据修改")
    @PostMapping("/modify/{vehicleCertId}")
    public ResponseMessage updTemplate(@PathVariable(value = "vehicleCertId") String vehicleCertId, @NotNull @RequestBody Map<String, String> params) {
        VehicleCertEO vehicleCertEO = vehicleCertEOService.getById(vehicleCertId);
        if (vehicleCertEO == null) {
            logger.error("VEHICLE_CERT的数据丢失：id=", vehicleCertId);
            return Result.error("一致性证书数据丢失");
        }
        YzxTemplateDataEO templateDataEO = service.getById(vehicleCertId);
        if (templateDataEO == null) {
            logger.error("YZX_TEMPLATE_DATA的数据丢失：id=" + vehicleCertId);
            return Result.error("一致性证书模板数据丢失");
        }
        PrintStatus printStatus = PrintStatus.getByValue(templateDataEO.getPrintStatus());
        if (PrintStatus.PRINTED == printStatus) {
            return Result.error("一致性证书已打印，无法修改");
        }
        if (ConfirmStatus.CONFIRMED.getValue().equals(templateDataEO.getConfirmStatus())) {
            return Result.error("一致性证书已确认，无法修改");
        }

        TemplateType templateType = TemplateType.getByValue(templateDataEO.getTemplateType());
        switch (templateType) {
            case TEMPLATE_1:
                YzxTemplateHandler.updYzxTemplate_1(templateDataEO, params);
                break;
            case TEMPLATE_2:
                YzxTemplateHandler.updYzxTemplate_2(templateDataEO, params);
                break;
            case TEMPLATE_3:
                YzxTemplateHandler.updYzxTemplate_3(templateDataEO, params);
                break;
            case TEMPLATE_4:
                YzxTemplateHandler.updYzxTemplate_4(templateDataEO, params);
                break;
            case TEMPLATE_5:
                YzxTemplateHandler.updYzxTemplate_5(templateDataEO, params);
                break;
            case TEMPLATE_6:
                YzxTemplateHandler.updYzxTemplate_6(templateDataEO, params);
                break;
            case TEMPLATE_7:
                YzxTemplateHandler.updYzxTemplate_7(templateDataEO, params);
                break;
            case TEMPLATE_8:
                YzxTemplateHandler.updYzxTemplate_8(templateDataEO, params);
                break;
            case TEMPLATE_9:
                YzxTemplateHandler.updYzxTemplate_9(templateDataEO, params);
                break;
            case TEMPLATE_10:
                YzxTemplateHandler.updYzxTemplate_10(templateDataEO, params);
                break;
            case TEMPLATE_11:
                YzxTemplateHandler.updYzxTemplate_11(templateDataEO, params);
                break;
            case TEMPLATE_12:
                YzxTemplateHandler.updYzxTemplate_12(templateDataEO, params);
                break;
            case TEMPLATE_13:
                YzxTemplateHandler.updYzxTemplate_13(templateDataEO, params);
                break;
            default:
                return Result.error("模板不存在");
        }
        templateDataEO.setUpdateTime(new Date())
                .setUpdater(UserUtil.getCurrentUserId());
        service.updateById(templateDataEO);
        return Result.success();
    }

    @TableAuthCheck(service = IVehicleCertEOService.class, paramName = "vehicleCertId")
    @ApiOperation(value = "|YzxTemplateDataEO|一致性模板已打印详细数据修改")
    @PostMapping("/modify2/{vehicleCertId}")
    public ResponseMessage updTemplate2(@PathVariable(value = "vehicleCertId") String vehicleCertId, @NotNull @RequestBody Map<String, String> params) {
        VehicleCertEO vehicleCertEO = vehicleCertEOService.getById(vehicleCertId);
        if (vehicleCertEO == null) {
            logger.error("VEHICLE_CERT的数据丢失：id=", vehicleCertId);
            return Result.error("一致性证书数据丢失");
        }
        YzxTemplateDataEO templateDataEO = service.getById(vehicleCertId);
        if (templateDataEO == null) {
            logger.error("YZX_TEMPLATE_DATA的数据丢失：id=" + vehicleCertId);
            return Result.error("一致性证书模板数据丢失");
        }
        PrintStatus printStatus = PrintStatus.getByValue(templateDataEO.getPrintStatus());
        if (PrintStatus.PRINTED != printStatus) {
            return Result.error("一致性证书打印状态错误");
        }

        TemplateType templateType = TemplateType.getByValue(templateDataEO.getTemplateType());
        switch (templateType) {
            case TEMPLATE_1:
                YzxTemplateHandler.updYzxTemplate_1(templateDataEO, params);
                break;
            case TEMPLATE_2:
                YzxTemplateHandler.updYzxTemplate_2(templateDataEO, params);
                break;
            case TEMPLATE_3:
                YzxTemplateHandler.updYzxTemplate_3(templateDataEO, params);
                break;
            case TEMPLATE_4:
                YzxTemplateHandler.updYzxTemplate_4(templateDataEO, params);
                break;
            case TEMPLATE_5:
                YzxTemplateHandler.updYzxTemplate_5(templateDataEO, params);
                break;
            case TEMPLATE_6:
                YzxTemplateHandler.updYzxTemplate_6(templateDataEO, params);
                break;
            case TEMPLATE_7:
                YzxTemplateHandler.updYzxTemplate_7(templateDataEO, params);
                break;
            case TEMPLATE_8:
                YzxTemplateHandler.updYzxTemplate_8(templateDataEO, params);
                break;
            case TEMPLATE_9:
                YzxTemplateHandler.updYzxTemplate_9(templateDataEO, params);
                break;
            case TEMPLATE_10:
                YzxTemplateHandler.updYzxTemplate_10(templateDataEO, params);
                break;
            case TEMPLATE_11:
                YzxTemplateHandler.updYzxTemplate_11(templateDataEO, params);
                break;
            case TEMPLATE_12:
                YzxTemplateHandler.updYzxTemplate_12(templateDataEO, params);
                break;
            case TEMPLATE_13:
                YzxTemplateHandler.updYzxTemplate_13(templateDataEO, params);
                break;
            default:
                return Result.error("模板不存在");
        }
        templateDataEO.setPrintStatus(PrintStatus.NOT_PRINT.getValue())
                .setConfirmStatus(ConfirmStatus.NOT_CONFIRM.getValue())
                .setDeclareStatus(DeclareStatus.NOT_DECLARE.getValue())
                .setUpdateTime(new Date())
                .setUpdater(UserUtil.getCurrentUserId());
        service.updateById(templateDataEO);
        return Result.success();
    }

    @ApiOperation(value = "|YzxTemplateDataEO|模板类型选择列表数据")
    @GetMapping("/getSelectOptions/{vehicleCertId}")
    public ResponseMessage getTmpSelectOptions(@PathVariable(value = "vehicleCertId") String vehicleCertId) {
        VehicleCertEO vehicleCertEO = vehicleCertEOService.getById(vehicleCertId);
        if (vehicleCertEO == null) {
            logger.error("VEHICLE_CERT的数据丢失：id=", vehicleCertId);
            return Result.error("一致性证书数据丢失");
        }
        CocInfoamtionVo cocInfoamtion = yzxBaseDataEOService.getCocInfoamtion(vehicleCertEO.getYzxCertno());
        if (cocInfoamtion == null) {
            logger.error("TT_COC_INFOAMTION的数据丢失：COC_ID=", vehicleCertEO.getYzxCertno());
            return Result.error("一致性证书数据丢失");
        }
        List<TemplateType> types = Arrays.stream(TemplateType.values()).filter(x -> {
            return x.getAutoType().contains(cocInfoamtion.getAutoType()) && x.getCocAutoType().equals(cocInfoamtion.getCocAutoType());
        }).collect(Collectors.toList());
        return Result.success(types);
    }

    @ApiOperation(value = "|VehicleCertYzxVo|一致性证书导出")
    @PostMapping(value = "export")
    public ResponseMessage exportYzxTemplate(HttpServletRequest request, HttpServletResponse response, @RequestBody Query<VehicleCertYzxVo> query) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        String qyId = usermap.get("USERNAME").toString();
        QueryWrapper queryWrapper = query.createWrapper();
        List yzxTemplateExcelVoList = service.getYzxTemplateExcelVoList(queryWrapper, qyId);
        Workbook workbook = ExcelUtils.exportExcel(yzxTemplateExcelVoList, "一致性证书打印列表", "导出信息", YzxTemplateExcelVo.class, true);
        ExcelUtils.downLoadExcel("一致性证书打印信息.xls", response, workbook);
        return Result.success();
    }

    @ApiOperation(value = "|VehicleCertYzxVo|一致性申报信息导出")
    @PostMapping(value = "declare/export")
    public ResponseMessage exportYzxDeclare(HttpServletRequest request, HttpServletResponse response, @RequestBody Query<VehicleCertYzxVo> query) {
        Map<String, Object> usermap = (Map<String, Object>) request.getSession().getAttribute("usermap");
        String qyId = usermap.get("USERNAME").toString();
        QueryWrapper queryWrapper = query.createWrapper();
        List yzxDeclareExcelVoList = service.getYzxDeclareExcelVoList(queryWrapper, qyId);
        Workbook workbook = ExcelUtils.exportExcel(yzxDeclareExcelVoList, "一致性证书申报列表", "导出信息", YzxDeclareExcelVo.class, true);
        ExcelUtils.downLoadExcel("一致性证书申报信息.xls", response, workbook);
        return Result.success();
    }

    @ApiOperation(value = "获取身份认证随机数")
    @GetMapping(value = "getVinUploadRand")
    public ResponseMessage getRand() {
        try {
            return Result.success(WebServiceUtil.getIVinUploadService().getRand());
        } catch (WebServiceException e) {
            logger.error("获取身份认证随机数失败", e);
            return Result.error("获取身份认证信息失败");
        }
    }

    @ApiOperation(value = "一致性申报")
    @PostMapping(value = "vinUpload", consumes = APPLICATION_JSON_VALUE)
    public ResponseMessage vinUpload(@Validated @RequestBody VinUploadVo vinUploadVo) {
        Map<String, VehicleCertEO> vehicleCertEOMap = vehicleCertEOService.listByIds(vinUploadVo.getIdList()).stream().collect(Collectors.toMap(VehicleCertEO::getId, VehicleCertEO -> VehicleCertEO));
        Collection<YzxTemplateDataEO> templateDataEOS = service.listByIds(vinUploadVo.getIdList());
        Map<String, YzxTemplateDataEO> templateDataEOMap = templateDataEOS.stream().collect(Collectors.toMap(YzxTemplateDataEO::getId, YzxTemplateDataEO -> YzxTemplateDataEO));
        List<VinBean> vinBeanList = new ArrayList<>();
        for (String vehicleCertId : vinUploadVo.getIdList()) {
            VehicleCertEO vehicleCertEO = vehicleCertEOMap.get(vehicleCertId);
            YzxTemplateDataEO templateDataEO = templateDataEOMap.get(vehicleCertId);
            if (vehicleCertEO == null || templateDataEO == null) {
                logger.error("车辆认证ID为" + vehicleCertId + "的数据丢失");
                return Result.error("车辆一致性数据丢失");
            }
            VinBean vinBean = new VinBean();
            vinBean.setVinCode(vehicleCertEO.getVin())
                    .setCocCode(templateDataEO.getCocId())
                    .setColor(vehicleCertEO.getVehicleColor())
                    .setEngineNo(templateDataEO.getDriveNo())
                    .setProductDate(String.valueOf(vehicleCertEO.getProduceDate().getTime()))
                    .setRemark(templateDataEO.getRemark());
            vinBeanList.add(vinBean);
        }
        IVinUploadService vinUploadService = WebServiceUtil.getIVinUploadService();
        try {
            String authJsonStr = vinUploadService.auth(vinUploadVo.getRand(), vinUploadVo.getSign());
            logger.info("一致性申报身份认证返回结果：" + authJsonStr);
            VinUploadRes authErrCd = VinUploadRes.getByValue(authJsonStr);
            if (authErrCd != VinUploadRes.NONE) {
                logger.error("一致性申报身份认证失败errCd=" + authErrCd.getValue() + ",message=" + authErrCd.getName());
                return Result.error("一致性申报身份认证失败：" + authErrCd.getName());
            }
            List<ManuRelationBean> manuRelationBeanList = objectMapper.readValue(authJsonStr, new TypeReference<List<ManuRelationBean>>() {
            });
            String manuRelationId = manuRelationBeanList.get(0).getManuRelationId();
            String vinUploadJsonStr = vinUploadService.upload(vinUploadVo.getRand(), manuRelationId, objectMapper.writeValueAsString(vinBeanList));
            logger.info("一致性申报上传返回结果：" + vinUploadJsonStr);
            Map<String, String> resMap = objectMapper.readValue(vinUploadJsonStr, new TypeReference<HashMap<String, String>>() {
            });
            VinUploadRes vinUploadRes = VinUploadRes.getByValue(resMap.get("status"));
            switch (vinUploadRes) {
                case CD_100:
                    Date sysDate = new Date();
                    String userId = UserUtil.getCurrentUserId();
                    templateDataEOS.forEach(x -> {
                        x.setDeclareStatus(DeclareStatus.DECLARED.getValue())
                                .setUpdateTime(sysDate)
                                .setUpdater(userId);
                    });
                    service.updateBatchById(templateDataEOS);
                    break;
                case CD_200:
                    StringBuilder errMsg = new StringBuilder();
                    vehicleCertEOMap.values().forEach(x -> {
                        String errCode = resMap.get(x.getVin());
                        if (StringUtils.isNoneBlank(errCode)) {
                            VinUploadRes errInfo = VinUploadRes.getByValue(errCode);
                            errMsg.append("车辆识别号").append(x.getVin()).append("上报失败,原因：").append(errInfo.getName()).append(";\n");
                        }
                    });
                    logger.error(errMsg.toString());
                    return Result.error(errMsg.toString());
                default:
                    logger.error("一致性申报失败errCd=" + vinUploadRes.getValue() + ",message=" + vinUploadRes.getName());
                    return Result.error("一致性申报失败");
            }
        } catch (WebServiceException wse) {
            logger.error("一致性申报失败", wse);
            return Result.error("一致性申报失败");
        } catch (IOException e) {
            logger.error("一致性申报失败", e);
            return Result.error("一致性申报失败");
        }
        return Result.success();
    }

    @Override
    protected IBaseService<YzxTemplateDataEO> getService() {

        return service;
    }

    @Override
    protected void verifyQuery(Query<YzxTemplateDataEO> queryItem) {

    }

    @Override
    protected void verifySaveData(YzxTemplateDataEO data) {

    }

    @Override
    protected void verifyUpdateData(YzxTemplateDataEO data) {

    }

    @Override
    protected void verifyDeleteId(Serializable... ids) {

    }

}
